﻿using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace Feel.common
{
    class GlassDildo : MVRScript
    {
        private static float MaxWobble = 0.09f;
        private static float WobbleSpeed = 5.0f;
        private static float RecoveryRate = 5f;

        private static Vector3 prevPos;
        private static Vector3 prevRot;
        private static float wobbleAmountToAddX;
        private static float wobbleAmountToAddZ;

        private JSONStorableStringChooser _person_list;
        private UIDynamicPopup _person_select_pop;

        private Transform _dildo;
        private Transform _liquid_root;
        private Transform _Inglass_root;
        private Rigidbody _labiaTrigger;
        private Rigidbody _vaginaTrigger;
        private FreeControllerV3 _thisControl;

        protected JSONStorableFloat lockAngleX;
        protected JSONStorableFloat lockAngleY;
        protected JSONStorableFloat lockAngleZ;
        protected JSONStorableFloat lockPosX;
        protected JSONStorableFloat lockPosY;
        protected JSONStorableFloat lockPosZ;

        private JSONStorableFloat _liquid_height;
        private JSONStorableBool _person_lock;
        private JSONStorableColor _ball_color;
        private JSONStorableColor _liquid_color;

        private bool _is_init = false;
        private int _initLoops = 0;
        private readonly int _maxInitLoops = 1000;

        public override void Init()
        {
            try
            {
                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("GlassDildo - Not a usable environment." + containingAtom.type);
                    return;
                }

                _thisControl = containingAtom.freeControllers.First(fc => fc.name == "control");

                _liquid_height = new JSONStorableFloat("Liquid Height", 0.5f, OnLiquidHeight, 0f, 1f);
                RegisterFloat(_liquid_height);
                CreateSlider(_liquid_height);

                var btn = CreateButton("Person Update");
                btn.button.onClick.AddListener(() =>
                {
                    UpdateAtomData();
                });

                _person_list = new JSONStorableStringChooser("Person Choice", null, null, "", (string target) =>
                {
                });

                _person_list.storeType = JSONStorableParam.StoreType.Physical;
                RegisterStringChooser(_person_list);

                _person_select_pop = CreateScrollablePopup(_person_list, false);
                _person_select_pop.popupPanelHeight = 800f;
                _person_select_pop.popup.onOpenPopupHandlers += () => {};
                _person_select_pop.label = "Person:";

                _person_lock = new JSONStorableBool("Person Lock", false);
                RegisterBool(_person_lock);
                CreateToggle(_person_lock);

                lockAngleX = new JSONStorableFloat("lock x rotation", 0f, OnChangeData, -180f, 180f);
                RegisterFloat(lockAngleX);
                CreateSlider(lockAngleX, true);
                lockAngleY = new JSONStorableFloat("lock y rotation", 0f, OnChangeData, -180f, 180f);
                RegisterFloat(lockAngleY);
                CreateSlider(lockAngleY, true);
                lockAngleZ = new JSONStorableFloat("lock z rotation", 0f, OnChangeData, -180f, 180f);
                RegisterFloat(lockAngleZ);
                CreateSlider(lockAngleZ, true);
                lockPosX = new JSONStorableFloat("lock x position", 0f, OnChangeData, -1f, 1f);
                RegisterFloat(lockPosX);
                CreateSlider(lockPosX, true);
                lockPosY = new JSONStorableFloat("lock y position", -0.4f, OnChangeData, -1f, 1f);
                RegisterFloat(lockPosY);
                CreateSlider(lockPosY, true);
                lockPosZ = new JSONStorableFloat("lock z position", 0f, OnChangeData, -1f, 1f);
                RegisterFloat(lockPosZ);
                CreateSlider(lockPosZ, true);

                HSVColor color = new HSVColor();
                color.H = 1f;
                color.S = 0f;
                color.V = 1f;
                _ball_color = new JSONStorableColor("Bag Shell Color", color, OnChangeBallColor);
                RegisterColor(_ball_color);
                CreateColorPicker(_ball_color);


            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void OnChangeBallColor(JSONStorableColor color)
        {
            if(_Inglass_root != null)
            {
                Renderer renderer = _Inglass_root.GetComponent<Renderer>();
                Color ma_color = Color.HSVToRGB(color.val.H, color.val.S, color.val.V);
                renderer.material.color = ma_color;
            }
        }

        void OnChangeData(float value)
        {

        }

        private void UpdateAtomData()
        {
            List<string> atomChoices = new List<string>();
            Atom atom;
            foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
            {
                atom = SuperController.singleton.GetAtomByUid(atomUID);
                JSONStorable geometry = atom.GetStorableByID("geometry");
                if (geometry != null)
                {
                    atomChoices.Add(atomUID);
                }
            }

            _person_list.choices = atomChoices;
        }

        private void OnLiquidHeight(float value)
        {
            if (_liquid_root == null) return;
            Renderer renderer = _liquid_root.GetComponent<Renderer>();
            float n = 0.65f - (value * 0.32f);
            //n = 0.5f;
            renderer.material.SetFloat("_FillAmount", n);
        }

        void Update()
        {
            if(_is_init)
            {
                Renderer render = _liquid_root.GetComponent<Renderer>();
                UpdateLiquidPhysics(_liquid_root, render);

            }
            if (_initLoops < _maxInitLoops)
            {
                _initLoops++;
                FirstInit();
            }
            else
            {
                if (_initLoops == 1001)
                {
                    SuperController.LogError("GlassDildo Load Failure " + _initLoops);
                    _initLoops += 1;
                }
            }
        }

        void FixedUpdate()
        {
            bool isl = SuperController.singleton.isLoading;
            if (!isl)
            {
                if (_person_lock.val && _person_list.val != null && _person_list.val != "")
                {
                    Atom target_girl = SuperController.singleton.GetAtomByUid(_person_list.val);
                    if (target_girl != null)
                    {
                        _labiaTrigger = target_girl.rigidbodies.First(rb => rb.name == "LabiaTrigger");
                        _vaginaTrigger = target_girl.rigidbodies.First(rb => rb.name == "VaginaTrigger");

                        _thisControl.transform.position = _labiaTrigger.transform.position;
                        _thisControl.transform.rotation = Quaternion.LookRotation(_vaginaTrigger.transform.position - _labiaTrigger.transform.position, _labiaTrigger.transform.up);
                        _thisControl.transform.Rotate(new Vector3(90f, 0f, 0f));

                        Vector3 offsetPosition = new Vector3(lockPosX.val / 5f, lockPosY.val / 5f, lockPosZ.val / 5f);
                        _thisControl.transform.Translate(offsetPosition, _thisControl.transform);

                        _thisControl.transform.Rotate(lockAngleX.val, lockAngleY.val, lockAngleZ.val);

                    }

                }
                
            }
        }

        private void FirstInit()
        {
            bool isl = SuperController.singleton.isLoading;
            if (isl) return;

            _dildo = getObjectRoot(containingAtom.reParentObject.transform, "glass_dildo");
            if (_dildo != null)
            {
                _liquid_root = getChildRoot(_dildo, "Liquid.001");
                _Inglass_root = getChildRoot(_dildo, "InGlass");
                UpdateAtomData();

                _is_init = true;
                _initLoops += 1001;
            }

        }

        public static Transform getObjectRoot(Transform parent, string rootName)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(rootName))
                {
                    return child;
                }
                else
                {
                    Transform childSearch = getObjectRoot(child, rootName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        public static Transform getChildRoot(Transform parent, string searchName)
        {
            foreach (Transform child in parent)
            {
                if (child.name == searchName)
                {

                    return child;
                }
                else
                {
                    Transform childSearch = getChildRoot(child, searchName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        public void UpdateLiquidPhysics(Transform LiquidRoot, Renderer rend)
        {
            wobbleAmountToAddX = Mathf.Lerp(wobbleAmountToAddX, 0, Time.deltaTime * RecoveryRate);
            wobbleAmountToAddZ = Mathf.Lerp(wobbleAmountToAddZ, 0, Time.deltaTime * RecoveryRate);

            // make a sine wave of the decreasing wobble
            float wobbleAmountX = wobbleAmountToAddX * Mathf.Sin(WobbleSpeed * Time.time);
            float wobbleAmountZ = wobbleAmountToAddZ * Mathf.Sin(WobbleSpeed * Time.time);

            // send it to the shader
            rend.material.SetFloat("_WobbleX", wobbleAmountX);
            rend.material.SetFloat("_WobbleZ", wobbleAmountZ);

            // Move Speed
            Vector3 moveSpeed = (prevPos - LiquidRoot.position) / Time.deltaTime;
            Vector3 rotationDelta = LiquidRoot.rotation.eulerAngles - prevRot;

            // add clamped speed to wobble
            wobbleAmountToAddX += Mathf.Clamp((moveSpeed.x + (rotationDelta.z * 0.5f)) * MaxWobble, -MaxWobble, MaxWobble);
            wobbleAmountToAddZ += Mathf.Clamp((moveSpeed.z + (rotationDelta.x * 0.5f)) * MaxWobble, -MaxWobble, MaxWobble);

            // save the last position
            prevPos = LiquidRoot.position;
            prevRot = LiquidRoot.rotation.eulerAngles;

        }
    }
}
